@@ -4,9 +4,10 @@
 #' around [expand()], [dplyr::full_join()] and [replace_na()] that's useful for
 #' completing missing combinations of data.
 #'
-#' @details
-#' With grouped data frames, `complete()` operates _within_ each group. Because
-#' of this, you cannot complete a grouping column.
+#' @section Grouped data frames:
+#' With grouped data frames created by [dplyr::group_by()], `complete()`
+#' operates _within_ each group. Because of this, you cannot complete a grouping
+#' column.
 #'
 #' @inheritParams expand
 #' @param fill A named list that for each variable supplies a single value to
@@ -17,8 +18,6 @@
 #'   missing values.
 #' @export
 #' @examples
-#' library(dplyr, warn.conflicts = FALSE)
-#'
 #' df <- tibble(
 #'   group = c(1:2, 1, 2),
 #'   item_id = c(1:2, 2, 3),
@@ -28,59 +27,53 @@
 #' )
 #' df
 #'
+#' # Combinations --------------------------------------------------------------
 #' # Generate all possible combinations of `group`, `item_id`, and `item_name`
 #' # (whether or not they appear in the data)
-#' complete(df, group, item_id, item_name)
+#' df |> complete(group, item_id, item_name)
 #'
 #' # Cross all possible `group` values with the unique pairs of
 #' # `(item_id, item_name)` that already exist in the data
-#' complete(df, group, nesting(item_id, item_name))
+#' df |> complete(group, nesting(item_id, item_name))
 #'
 #' # Within each `group`, generate all possible combinations of
 #' # `item_id` and `item_name` that occur in that group
-#' df %>%
-#'   group_by(group) %>%
+#' df |>
+#'   dplyr::group_by(group) |>
 #'   complete(item_id, item_name)
 #'
-#' # You can also choose to fill in missing values. By default, both implicit
-#' # (new) and explicit (pre-existing) missing values are filled.
-#' complete(
-#'   df,
-#'   group,
-#'   nesting(item_id, item_name),
-#'   fill = list(value1 = 0, value2 = 99)
-#' )
+#' # Supplying values for new rows ---------------------------------------------
+#' # Use `fill` to replace NAs with some value. By default, affects both new
+#' # (implicit) and pre-existing (explicit) missing values.
+#' df |>
+#'   complete(
+#'     group,
+#'     nesting(item_id, item_name),
+#'     fill = list(value1 = 0, value2 = 99)
+#'   )
 #'
-#' # You can limit the fill to only implicit missing values by setting
-#' # `explicit` to `FALSE`
-#' complete(
-#'   df,
-#'   group,
-#'   nesting(item_id, item_name),
-#'   fill = list(value1 = 0, value2 = 99),
-#'   explicit = FALSE
-#' )
-complete <- function(data,
-                     ...,
-                     fill = list(),
-                     explicit = TRUE) {
+#' # Limit the fill to only the newly created (i.e. previously implicit)
+#' # missing values with `explicit = FALSE`
+#' df |>
+#'   complete(
+#'     group,
+#'     nesting(item_id, item_name),
+#'     fill = list(value1 = 0, value2 = 99),
+#'     explicit = FALSE
+#'   )
+complete <- function(data, ..., fill = list(), explicit = TRUE) {
   UseMethod("complete")
 }
 
 #' @export
-complete.data.frame <- function(data,
-                                ...,
-                                fill = list(),
-                                explicit = TRUE) {
-  if (!is_bool(explicit)) {
-    abort("`explicit` must be a single `TRUE` or `FALSE`.")
-  }
+complete.data.frame <- function(data, ..., fill = list(), explicit = TRUE) {
+  check_bool(explicit)
 
   out <- expand(data, ...)
   names <- names(out)
 
   if (length(names) > 0L) {
-    out <- dplyr::full_join(out, data, by = names)
+    out <- dplyr::full_join(out, data, by = names, multiple = "all")
   } else {
     # Avoid joining the 1x0 result from `expand()` with `data`.
     # That causes issues when `data` has zero rows.
@@ -100,18 +93,22 @@ complete.data.frame <- function(data,
 }
 
 #' @export
-complete.grouped_df <- function(data,
-                                ...,
-                                fill = list(),
-                                explicit = TRUE) {
-  dplyr::summarise(
+complete.grouped_df <- function(data, ..., fill = list(), explicit = TRUE) {
+  out <- dplyr::reframe(
     data,
     complete(
-      data = dplyr::cur_data(),
+      data = dplyr::pick(everything()),
       ...,
       fill = fill,
       explicit = explicit
-    ),
-    .groups = "keep"
+    )
+  )
+
+  drop <- dplyr::group_by_drop_default(data)
+
+  dplyr::group_by(
+    out,
+    !!!dplyr::groups(data),
+    .drop = drop
   )
 }
