@@ -6,8 +6,8 @@
 #' Development on `gather()` is complete, and for new code we recommend
 #' switching to `pivot_longer()`, which is easier to use, more featureful, and
 #' still under active development.
-#' `df %>% gather("key", "value", x, y, z)` is equivalent to
-#' `df %>% pivot_longer(c(x, y, z), names_to = "key", values_to = "value")`
+#' `df |> gather("key", "value", x, y, z)` is equivalent to
+#' `df |> pivot_longer(c(x, y, z), names_to = "key", values_to = "value")`
 #'
 #' See more details in `vignette("pivot")`.
 #'
@@ -60,17 +60,16 @@
 #' @inheritParams gather_
 #' @export
 #' @examples
-#' library(dplyr)
 #' # From https://stackoverflow.com/questions/1181060
 #' stocks <- tibble(
-#'   time = as.Date('2009-01-01') + 0:9,
+#'   time = as.Date("2009-01-01") + 0:9,
 #'   X = rnorm(10, 0, 1),
 #'   Y = rnorm(10, 0, 2),
 #'   Z = rnorm(10, 0, 4)
 #' )
 #'
 #' gather(stocks, "stock", "price", -time)
-#' stocks %>% gather("stock", "price", -time)
+#' stocks |> gather("stock", "price", -time)
 #'
 #' # get first observation for each Species in iris data -- base R
 #' mini_iris <- iris[c(1, 51, 101), ]
@@ -79,31 +78,36 @@
 #'        Sepal.Length, Sepal.Width, Petal.Length, Petal.Width)
 #' # same result but less verbose
 #' gather(mini_iris, key = "flower_att", value = "measurement", -Species)
-#'
-#' # repeat iris example using dplyr and the pipe operator
-#' library(dplyr)
-#' mini_iris <-
-#'   iris %>%
-#'   group_by(Species) %>%
-#'   slice(1)
-#' mini_iris %>% gather(key = "flower_att", value = "measurement", -Species)
-gather <- function(data, key = "key", value = "value", ...,
-                   na.rm = FALSE, convert = FALSE, factor_key = FALSE) {
-  ellipsis::check_dots_unnamed()
+gather <- function(
+  data,
+  key = "key",
+  value = "value",
+  ...,
+  na.rm = FALSE,
+  convert = FALSE,
+  factor_key = FALSE
+) {
+  check_dots_unnamed()
   UseMethod("gather")
 }
 #' @export
-gather.data.frame <- function(data, key = "key", value = "value", ...,
-                              na.rm = FALSE, convert = FALSE,
-                              factor_key = FALSE) {
-  key_var <- as_string(ensym2(key))
-  value_var <- as_string(ensym2(value))
+gather.data.frame <- function(
+  data,
+  key = "key",
+  value = "value",
+  ...,
+  na.rm = FALSE,
+  convert = FALSE,
+  factor_key = FALSE
+) {
+  key_var <- as_string(ensym(key))
+  value_var <- as_string(ensym(value))
 
   quos <- quos(...)
   if (is_empty(quos)) {
     gather_vars <- setdiff(names(data), c(key_var, value_var))
   } else {
-    gather_vars <- unname(tidyselect::vars_select(tbl_vars(data), !!! quos))
+    gather_vars <- unname(tidyselect::vars_select(tbl_vars(data), !!!quos))
   }
 
   if (is_empty(gather_vars)) {
@@ -180,9 +184,13 @@ normalize_melt_arguments <- function(data, measure.ind) {
 }
 
 all_identical <- function(xs) {
-  if (length(xs) <= 1) return(TRUE)
+  if (length(xs) <= 1) {
+    return(TRUE)
+  }
   for (i in seq(2, length(xs))) {
-    if (!identical(xs[[1]], xs[[i]])) return(FALSE)
+    if (!identical(xs[[1]], xs[[i]])) {
+      return(FALSE)
+    }
   }
   TRUE
 }
