@@ -16,7 +16,7 @@ id <- function(.variables, drop = FALSE) {
   # Calculate dimensions
   ndistinct <- map_dbl(ids, attr, "n")
   n <- prod(ndistinct)
-  if (n > 2 ^ 31) {
+  if (n > 2^31) {
     # Too big for integers, have to use strings, which will be much slower :(
 
     char_id <- do.call("paste", c(ids, sep = "\r"))
@@ -29,7 +29,6 @@ id <- function(.variables, drop = FALSE) {
   }
   attr(res, "n") <- n
 
-
   if (drop) {
     id_var(res, drop = TRUE)
   } else {
@@ -38,11 +37,14 @@ id <- function(.variables, drop = FALSE) {
 }
 
 id_var <- function(x, drop = FALSE) {
-  if (!is_null(attr(x, "n")) && !drop) return(x)
+  if (!is_null(attr(x, "n", exact = TRUE)) && !drop) {
+    return(x)
+  }
 
   if (is.factor(x) && !drop) {
-    id <- as.integer(addNA(x, ifany = TRUE))
-    n <- length(levels(x))
+    x_na <- addNA(x, ifany = TRUE)
+    id <- as.integer(x_na)
+    n <- length(levels(x_na))
   } else if (length(x) == 0) {
     id <- integer()
     n <- 0L
