@@ -17,10 +17,17 @@ full_seq <- function(x, period, tol = 1e-6) {
 
 #' @export
 full_seq.numeric <- function(x, period, tol = 1e-6) {
+  check_number_decimal(period)
+  check_number_decimal(tol, min = 0)
+
   rng <- range(x, na.rm = TRUE)
-  if (any(((x - rng[1]) %% period > tol) &
-          (period - (x - rng[1]) %% period > tol))) {
-    stop("`x` is not a regular sequence.", call. = FALSE)
+  if (
+    any(
+      ((x - rng[1]) %% period > tol) &
+        (period - (x - rng[1]) %% period > tol)
+    )
+  ) {
+    cli::cli_abort("{.arg x} is not a regular sequence.")
   }
 
   # in cases where the last element is within tolerance, pad it so that
@@ -42,7 +49,6 @@ full_seq.POSIXct <- function(x, period, tol = 1e-6) {
   restore(x, full_seq(as.numeric(x), period, tol))
 }
 
-
 restore <- function(old, new) {
   mostattributes(new) <- attributes(old)
   new
