@@ -6,12 +6,14 @@
 #' @param data A data frame, tibble, or grouped tibble.
 #' @param weights A vector of weights. Evaluated in the context of `data`;
 #'   supports quasiquotation.
+#' @param ... Additional arguments passed on to methods.
 #' @param .id Supply a string to create a new variable which gives a unique
 #'   identifier for each created row.
-#' @param .remove If `TRUE`, and `weights` is a `single`
+#' @param .remove If `TRUE`, and `weights` is the name of a column in `data`,
+#'   then this column is removed.
 #' @export
 #' @examples
-#' df <- tibble::tibble(x = c("a", "b"), n = c(1, 2))
+#' df <- tibble(x = c("a", "b"), n = c(1, 2))
 #' uncount(df, n)
 #' uncount(df, n, .id = "id")
 #'
@@ -20,16 +22,28 @@
 #'
 #' # Or expressions
 #' uncount(df, 2 / n)
-uncount <- function(data, weights, .remove = TRUE, .id = NULL) {
+uncount <- function(data, weights, ..., .remove = TRUE, .id = NULL) {
+  check_dots_used()
+  UseMethod("uncount")
+}
+
+#' @export
+uncount.data.frame <- function(data, weights, ..., .remove = TRUE, .id = NULL) {
+  check_bool(.remove)
+  check_name(.id, allow_null = TRUE)
+
   weights_quo <- enquo(weights)
-  w <- dplyr::pull(dplyr::mutate(data, `_weight` = !! weights_quo))
+  w <- dplyr::pull(dplyr::mutate(data, `_weight` = !!weights_quo))
 
-  if (!is.numeric(w)) {
-    stop("`weights` must evaluate to a numeric vector", call. = FALSE)
-  }
-
-  out <- data[rep(seq_nrow(data), w), , drop = FALSE]
+  out <- vec_rep_each(
+    data,
+    w,
+    error_call = current_env(),
+    times_arg = "weights"
+  )
 
+  # NOTE it was decided to also remove grouping variables as there is no clear
+  # best answer. See https://github.com/tidyverse/tidyr/pull/1070
   if (.remove && quo_is_symbol(weights_quo)) {
     out[[as_string(get_expr(weights_quo))]] <- NULL
   }
