@@ -26,19 +26,21 @@ test_that("can control output column name", {
 
 test_that("nest doesn't include grouping vars in nested data", {
   df <- tibble(x = c(1, 1, 1), y = 1:3)
-  out <- df %>% dplyr::group_by(x) %>% nest_legacy()
+  out <- df |>
+    dplyr::group_by(x) |>
+    nest_legacy()
   expect_equal(out$data[[1]], tibble(y = 1:3))
 })
 
 test_that("can restrict variables in grouped nest", {
-  df <- tibble(x = 1, y = 2, z = 3) %>% dplyr::group_by(x)
-  out <- df %>% nest_legacy(y)
+  df <- tibble(x = 1, y = 2, z = 3) |> dplyr::group_by(x)
+  out <- df |> nest_legacy(y)
   expect_equal(names(out$data[[1]]), "y")
 })
 
 test_that("puts data into the correct row", {
   df <- tibble(x = 1:3, y = c("B", "A", "A"))
-  out <- nest_legacy(df, x) %>% dplyr::filter(y == "B")
+  out <- nest_legacy(df, x) |> dplyr::filter(y == "B")
   expect_equal(out$data[[1]]$x, 1)
 })
 
@@ -79,13 +81,13 @@ test_that("nesting works for empty data frames", {
 })
 
 test_that("tibble conversion occurs in the `nest.data.frame()` method", {
-  tbl <- mtcars %>% nest_legacy(-am, -cyl)
+  tbl <- mtcars |> nest_legacy(-am, -cyl)
   expect_s3_class(tbl, "tbl_df")
   expect_s3_class(tbl$data[[1L]], "tbl_df")
 })
 
 test_that("nest_legacy() does not preserve grouping", {
-  df <- tibble(x = c(1, 1, 2), y = 1:3) %>% dplyr::group_by(x)
+  df <- tibble(x = c(1, 1, 2), y = 1:3) |> dplyr::group_by(x)
   out <- nest_legacy(df)
   expect_false(inherits(out, "grouped_df"))
 })
@@ -97,7 +99,7 @@ test_that("unnesting combines atomic vectors", {
   expect_equal(unnest_legacy(df)$x, 1:10)
 })
 
-test_that("unesting combines augmented vectors", {
+test_that("unnesting combines augmented vectors", {
   df <- tibble::tibble(x = as.list(as.factor(letters[1:3])))
   expect_equal(unnest_legacy(df)$x, factor(letters[1:3]))
 })
@@ -109,10 +111,12 @@ test_that("vector unnest preserves names", {
 })
 
 test_that("unnesting row binds data frames", {
-  df <- tibble(x = list(
-    tibble(x = 1:5),
-    tibble(x = 6:10)
-  ))
+  df <- tibble(
+    x = list(
+      tibble(x = 1:5),
+      tibble(x = 6:10)
+    )
+  )
   expect_equal(unnest_legacy(df)$x, 1:10)
 })
 
@@ -131,7 +135,10 @@ test_that("can unnest mixture of name and unnamed lists of same length", {
     y = list(y = 1:2),
     z = list(1:2)
   )
-  expect_identical(unnest_legacy(df), tibble(x = c("a","a"), y = c(1:2), z = c(1:2)))
+  expect_identical(
+    unnest_legacy(df),
+    tibble(x = c("a", "a"), y = c(1:2), z = c(1:2))
+  )
 })
 
 test_that("elements must all be of same type", {
@@ -141,20 +148,20 @@ test_that("elements must all be of same type", {
 
 test_that("can't combine vectors and data frames", {
   df <- tibble(x = list(1, tibble(1)))
-  expect_snapshot((expect_error(unnest_legacy(df))))
+  expect_snapshot(unnest_legacy(df), error = TRUE)
 })
 
 test_that("multiple columns must be same length", {
   df <- tibble(x = list(1), y = list(1:2))
-  expect_snapshot((expect_error(unnest_legacy(df))))
+  expect_snapshot(unnest_legacy(df), error = TRUE)
 
   df <- tibble(x = list(1), y = list(tibble(x = 1:2)))
-  expect_snapshot((expect_error(unnest_legacy(df))))
+  expect_snapshot(unnest_legacy(df), error = TRUE)
 })
 
 test_that("nested is split as a list (#84)", {
   df <- tibble(x = 1:3, y = list(1, 2:3, 4), z = list(5, 6:7, 8))
-  expect_warning(out <- unnest_legacy(df, y, z), NA)
+  expect_no_warning(out <- unnest_legacy(df, y, z))
   expect_equal(out$x, c(1, 2, 2, 3))
   expect_equal(out$y, unlist(df$y))
   expect_equal(out$z, unlist(df$z))
@@ -162,7 +169,7 @@ test_that("nested is split as a list (#84)", {
 
 test_that("unnest has mutate semantics", {
   df <- tibble(x = 1:3, y = list(1, 2:3, 4))
-  out <- df %>% unnest_legacy(z = map(y, `+`, 1))
+  out <- df |> unnest_legacy(z = map(y, `+`, 1))
 
   expect_equal(out$z, 2:5)
 })
@@ -175,7 +182,7 @@ test_that(".id creates vector of names for vector unnest", {
 })
 
 test_that(".id creates vector of names for grouped vector unnest", {
-  df <- tibble(x = 1:2, y = list(a = 1, b = 1:2)) %>%
+  df <- tibble(x = 1:2, y = list(a = 1, b = 1:2)) |>
     dplyr::group_by(x)
   out <- unnest_legacy(df, .id = "name")
 
@@ -183,20 +190,26 @@ test_that(".id creates vector of names for grouped vector unnest", {
 })
 
 test_that(".id creates vector of names for data frame unnest", {
-  df <- tibble(x = 1:2, y = list(
-    a = tibble(y = 1),
-    b = tibble(y = 1:2)
-  ))
+  df <- tibble(
+    x = 1:2,
+    y = list(
+      a = tibble(y = 1),
+      b = tibble(y = 1:2)
+    )
+  )
   out <- unnest_legacy(df, .id = "name")
 
   expect_equal(out$name, c("a", "b", "b"))
 })
 
 test_that(".id creates vector of names for grouped data frame unnest", {
-  df <- tibble(x = 1:2, y = list(
-    a = tibble(y = 1),
-    b = tibble(y = 1:2)
-  )) %>%
+  df <- tibble(
+    x = 1:2,
+    y = list(
+      a = tibble(y = 1),
+      b = tibble(y = 1:2)
+    )
+  ) |>
     dplyr::group_by(x)
   out <- unnest_legacy(df, .id = "name")
 
@@ -204,15 +217,15 @@ test_that(".id creates vector of names for grouped data frame unnest", {
 })
 
 test_that("can use non-syntactic names", {
-  out <- tibble("foo bar" = list(1:2, 3)) %>% unnest_legacy()
+  out <- tibble("foo bar" = list(1:2, 3)) |> unnest_legacy()
 
   expect_named(out, "foo bar")
 })
 
 test_that("sep combines column names", {
   ldf <- list(tibble(x = 1))
-  tibble(x = ldf, y = ldf) %>%
-    unnest_legacy(.sep = "_") %>%
+  tibble(x = ldf, y = ldf) |>
+    unnest_legacy(.sep = "_") |>
     expect_named(c("x_x", "y_x"))
 })
 
@@ -229,24 +242,24 @@ test_that("empty ... returns df if no list-cols", {
 
 test_that("can optional preserve list cols", {
   df <- tibble(x = list(3, 4), y = list("a", "b"))
-  rs <- df %>% unnest_legacy(x, .preserve = y)
+  rs <- df |> unnest_legacy(x, .preserve = y)
   expect_identical(rs, tibble(y = df$y, x = c(3, 4)))
 
   df <- tibble(x = list(c("d", "e")), y = list(1:2))
-  rs <- df %>% unnest_legacy(.preserve = y)
+  rs <- df |> unnest_legacy(.preserve = y)
   expect_identical(rs, tibble(y = rep(list(1:2), 2), x = c("d", "e")))
 })
 
 test_that("unnest drops list cols if expanding", {
   df <- tibble(x = 1:2, y = list(3, 4), z = list(5, 6:7))
-  out <- df %>% unnest_legacy(z)
+  out <- df |> unnest_legacy(z)
 
   expect_equal(names(out), c("x", "z"))
 })
 
 test_that("unnest keeps list cols if not expanding", {
   df <- tibble(x = 1:2, y = list(3, 4), z = list(5, 6:7))
-  out <- df %>% unnest_legacy(y)
+  out <- df |> unnest_legacy(y)
 
   expect_equal(names(out), c("x", "z", "y"))
 })
@@ -254,13 +267,16 @@ test_that("unnest keeps list cols if not expanding", {
 test_that("unnest respects .drop_lists", {
   df <- tibble(x = 1:2, y = list(3, 4), z = list(5, 6:7))
 
-  expect_equal(df %>% unnest_legacy(y, .drop = TRUE) %>% names(), c("x", "y"))
-  expect_equal(df %>% unnest_legacy(z, .drop = FALSE) %>% names(), c("x", "y", "z"))
+  expect_equal(df |> unnest_legacy(y, .drop = TRUE) |> names(), c("x", "y"))
+  expect_equal(
+    df |> unnest_legacy(z, .drop = FALSE) |> names(),
+    c("x", "y", "z")
+  )
 })
 
 test_that("grouping is preserved", {
-  df <- tibble(g = 1, x = list(1:3)) %>% dplyr::group_by(g)
-  rs <- df %>% unnest_legacy(x)
+  df <- tibble(g = 1, x = list(1:3)) |> dplyr::group_by(g)
+  rs <- df |> unnest_legacy(x)
 
   expect_equal(rs$x, 1:3)
   expect_equal(class(df), class(rs))
@@ -269,7 +285,10 @@ test_that("grouping is preserved", {
 
 test_that("unnesting zero row column preserves names", {
   df <- tibble(a = character(), b = character())
-  expect_equal(df %>% unnest_legacy(b), tibble(a = character(), b = character()))
+  expect_equal(
+    df |> unnest_legacy(b),
+    tibble(a = character(), b = character())
+  )
 })
 
 test_that("unnest_legacy() recognize ptype", {
